using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using MonitoringApp.Custom;
using System.Text.RegularExpressions;
using Gtk;
namespace MonitoringApp.Code.App
{
    public class Configuration
    {

        private static bool isLoadFirst = false;
        private static List<MyConfig> configLst;
        public static List<MyConfig> ConfigLst
        {
            get
            {
                if (configLst == null)
                {
                    try
                    {
                        if (!isLoadFirst)
                            return LoadFromInitParams();
                        else
                            return null;
                    }
                    catch (Exception ex)
                    {
                            LogManager.WriteErrorLog(ex);

                        return null;
                    }
                }
                return configLst;
            }
            set { configLst = value; }
        }

        /// <summary>
        /// Load Configuration file
        /// </summary>
        /// <returns></returns>
        private static List<MyConfig> LoadFromInitParams()
        {
            isLoadFirst = true;
            string _configPath = Path.Combine(Directory.GetCurrentDirectory(), "Configuration.txt");
            if (File.Exists(_configPath))
            {
                if (ConfigLst == null)
                    ConfigLst = new List<MyConfig>();
                else
                    return ConfigLst;
                using (StreamReader sr = new StreamReader(_configPath))
                {
                    int lineIndex = 1;
                    while (sr.Peek() > 0)
                    {
                        string str = sr.ReadLine().Trim();
                        if (str.Length == 0)
                            continue;
                        if (!str.Contains("[") || !str.Contains("]"))
                        {
                            LogManager.WriteErrorLog(string.Format("Config Error:'[' OR ']' Not Exist.At Line {0}.", lineIndex));
                            continue;
                        }
                        if (!str.Contains(":"))
                        {
                            LogManager.WriteErrorLog(string.Format("Config Error:':' Not Exist.At Line {0}.", lineIndex));
                            continue;
                        }
                        string[] names = str.Split(':');
                        str = str.Substring(str.IndexOf(':') + 1).Trim(']').Trim('[').Trim('\'');
                        string[] strConfigs = str.Split(',');
                        SetConfig(names, ConfigLst, strConfigs, lineIndex == 1);
                        lineIndex++;
                    }
                }
                return ConfigLst;
            }
            else
            {
                LogManager.WriteErrorLog(string.Format("can't find config file.Path:{0}.",_configPath));
                return null;
            }
        }
        /// <summary>
        /// Set Config
        /// </summary>
        /// <param name="name"></param>
        /// <param name="lst"></param>
        /// <param name="strConfigs"></param>
        private static void SetConfig(string[] name, List<MyConfig> lst, string[] strConfigs, bool isLineOne)
        {
            try
            {
                var mcConfig = (MonitoringConfigurations)Enum.Parse(typeof(MonitoringConfigurations), name[0], true);
                //Set Values
                for (int i = 0; i < strConfigs.Length; i++)
                {
                    string value = strConfigs[i].Trim('\'');
                    switch (mcConfig)
                    {
                        case MonitoringConfigurations.MonitoringDirectoryType:
                            {
                                #region MonitoringDirectoryType Defalut:immediate
                                switch (value.ToLower())
                                {
                                    case "immediate":
                                        {
                                            if (isLineOne)
                                            {
                                                MyConfig myConfig = new MyConfig();
                                                myConfig.MDType = MonitoringDirectoryType.Immediate;
                                                lst.Add(myConfig);
                                            }
                                            else
                                            {
                                                lst[i].MDType = MonitoringDirectoryType.Immediate;
                                            }
                                            break;
                                        }
                                    case "cycle":
                                        {
                                            if (isLineOne)
                                            {
                                                MyConfig myConfig = new MyConfig();
                                                myConfig.MDType = MonitoringDirectoryType.Cycle;
                                                lst.Add(myConfig);
                                            }
                                            else
                                            {
                                                lst[i].MDType = MonitoringDirectoryType.Cycle;
                                            }
                                            break;
                                        }
                                    default:
                                        {
                                            if (isLineOne)
                                            {
                                                MyConfig myConfig = new MyConfig();
                                                myConfig.MDType = MonitoringDirectoryType.Immediate;
                                                lst.Add(myConfig);
                                            }
                                            else
                                            {
                                                lst[i].MDType = MonitoringDirectoryType.Immediate;
                                            }
                                            break;
                                        }
                                }
                                #endregion
                                break;
                            }
                        case MonitoringConfigurations.Path:
                            #region Path
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.Path = value;
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].Path = value;
                            }
                            #endregion
                            break;
                        case MonitoringConfigurations.IncludeSubdirectories:
                            #region IncludeSubdirectories
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.IncludeSubdirectories = Convert.ToBoolean(value);
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].IncludeSubdirectories = Convert.ToBoolean(value);
                            }
                            #endregion
                            break;
                        case MonitoringConfigurations.Filter:
                            #region Filter
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.Filter = value;
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].Filter = value;
                            }
                            break;
                            #endregion
                        case MonitoringConfigurations.BackupPath:
                            #region BackupPath
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.BackupPath = value;
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].BackupPath = value;
                            }
                            #endregion
                            break;
                        case MonitoringConfigurations.TargetPath:
                            #region TargetPath
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.TargetPath = value;
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].TargetPath = value;
                            }
                            #endregion
                            break;
                        case MonitoringConfigurations.BackupExpired:
                            #region BackupExpired  Unit:days  Default:30
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.BackupExpired = Convert.ToInt32(value.Length == 0 ? "30" : value);
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].BackupExpired = Convert.ToInt32(value.Length == 0 ? "30" : value);
                            }
                            #endregion
                            break;
                        case MonitoringConfigurations.CycleMinutes:
                            #region CycleMinutes  Unit:Minute  Default:60
                            if (isLineOne)
                            {
                                MyConfig myConfig = new MyConfig();
                                myConfig.CycleMinutes = Convert.ToInt32(value.Length == 0 ? "60" : value);
                                lst.Add(myConfig);
                            }
                            else
                            {
                                lst[i].CycleMinutes = Convert.ToInt32(value.Length == 0 ? "60" : value);
                            }
                            #endregion
                            break;
                        default:
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                LogManager.WriteErrorLog(ex);
            }


        }

    }
}
