using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Gtk;

namespace MonitoringApp.Code.App
{
    public class CreateMDDir
    {
        public static object obj = new object();
        public static void CreateMDDirForInit()
        {
            //Load Configuration
            var configLst = Configuration.ConfigLst;
            if (configLst != null)
            {
                MDDirManager.DeleteBackupExpiredDirs();
                foreach (var myConfig in configLst)
                {
                    #region Check Path
                    if (string.IsNullOrEmpty(myConfig.Path))
                    {
                        LogManager.WriteErrorLog("Not Config \"MonitoringDirectory\" Element's Dir Path.");
                        continue;
                    }
                    if (!Directory.Exists(myConfig.Path))
                    {
                        LogManager.WriteErrorLog(string.Format("Directory '{0}' Not Exist.", myConfig.Path));
                        continue;
                    }
                    #endregion
                    switch (myConfig.MDType)
                    {
                        case MonitoringApp.Custom.MonitoringDirectoryType.Immediate:
                            CreateImmediateDir(myConfig);
                            break;
                        case MonitoringApp.Custom.MonitoringDirectoryType.Cycle:
                           
                            var taskTimer = new TaskTimer() { Enabled = true, AutoReset = true, Interval = myConfig.CycleMinutes * 60000, CurrentConfig = myConfig };
                            taskTimer.Elapsed += new System.Timers.ElapsedEventHandler(taskTimer_Elapsed);
                            break;
                        default:
                            break;
                    }
                }
            }
            else
            {
                LogManager.WriteErrorLog("Load Configuration Fail,Is NUll.");
                Application.Quit();
            }
        }

        static void taskTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            TaskTimer taskTimer = sender as TaskTimer;
            if (taskTimer == null)
            {
                LogManager.WriteErrorLog("TaskTimer is NULL.");
                return;
            }
            lock (obj)
            {
                try
                {
                    var backupPath = Path.Combine(taskTimer.CurrentConfig.BackupPath, string.Format(@"{0}-{1}\{2}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day));
                    if (!Directory.Exists(backupPath))
                    {
                        Directory.CreateDirectory(backupPath);
                    }
                    var targetPath = Path.Combine(taskTimer.CurrentConfig.TargetPath, string.Format(@"{0}-{1}\{2}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day));
                    if (!Directory.Exists(targetPath))
                    {
                        Directory.CreateDirectory(targetPath);
                    }
                    MDDirManager.CopyFolder(taskTimer.CurrentConfig.Path, backupPath);
                    DirectoryInfo backupDir = new DirectoryInfo(taskTimer.CurrentConfig.Path);
                    MDDirManager.CopyFolder(Path.Combine(backupPath, backupDir.Name), targetPath);
                    MDDirManager.DeleteChildrenFolderAndFiles(taskTimer.CurrentConfig.Path);
                    LogManager.WriteTraceLog(string.Format("Has Backup!SourcePath:{0},BackupPath:{1},TargetPath:{2}", taskTimer.CurrentConfig.Path, Path.Combine(backupPath, backupDir.Name), targetPath));
                }
                catch (Exception ex)
                {
                    LogManager.WriteErrorLog(ex);
                }
            }
        }


        private static void CreateImmediateDir(Custom.MyConfig myConfig)
        {
            FileSystemWatcher watcher = new FileSystemWatcher();
            watcher.Path = myConfig.Path;
            watcher.EnableRaisingEvents = true;
            watcher.IncludeSubdirectories = myConfig.IncludeSubdirectories;
            watcher.Filter = myConfig.Filter;
            //watcher.Created += new FileSystemEventHandler(watcher_Created);
            watcher.Changed += new FileSystemEventHandler(watcher_Changed);
            //watcher.Deleted += new FileSystemEventHandler(watcher_Deleted);
        }

        static void watcher_Deleted(object sender, FileSystemEventArgs e)
        {
        }

        static void watcher_Changed(object sender, FileSystemEventArgs e)
        {
            UploadFilesAndBackUp(e);
        }

        static void watcher_Created(object sender, FileSystemEventArgs e)
        {
            UploadFilesAndBackUp(e);
        }

        private static void UploadFilesAndBackUp(FileSystemEventArgs e)
        {
            try
            {
                if (Configuration.ConfigLst == null)
                    return;
                var myConfig = Configuration.ConfigLst.FirstOrDefault(p => e.FullPath.StartsWith(p.Path));
                if (myConfig == null)
                {
                    LogManager.WriteErrorLog("Can't find MDDir's Path in ConfigLst.myConfig is NULL.");
                    return;
                }
                if (string.IsNullOrEmpty(myConfig.BackupPath) || !Directory.Exists(myConfig.BackupPath))
                {
                    LogManager.WriteErrorLog("BackUp Path Not Exists!");
                    return;
                }
                var backupPath = Path.Combine(myConfig.BackupPath, string.Format(@"{0}-{1}\{2}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day));
                var targetPath = Path.Combine(myConfig.TargetPath, string.Format(@"{0}-{1}\{2}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day));
                if (!Directory.Exists(backupPath))
                    Directory.CreateDirectory(backupPath);
                if (!Directory.Exists(targetPath))
                    Directory.CreateDirectory(targetPath);
                //IF is File
                if (File.Exists(e.FullPath))
                {
                    lock (obj)
                    {
                        try
                        {
                            string _backFilePath = Path.Combine(backupPath, e.Name);
                            string _targetFilePath = Path.Combine(targetPath, e.Name);
                            File.Copy(e.FullPath, _backFilePath, true);
                            File.Copy(_backFilePath, _targetFilePath, true);
                            File.Delete(e.FullPath);
                            LogManager.WriteTraceLog(string.Format("Has Backup And Copy!SourcePath:{0},BackupPath:{1},TargetPath:{2}", e.FullPath, _backFilePath, _targetFilePath));
                        }
                        catch (Exception ex)
                        {
                            LogManager.WriteErrorLog(ex);
                        }
                    }
                }
                //if is Directory
                if (Directory.Exists(e.FullPath))
                {
                    lock (obj)
                    {
                        try
                        {
                            MDDirManager.CopyChildrenFolderAndFiles(e.FullPath, Path.Combine(backupPath, e.Name));
                            MDDirManager.CopyChildrenFolderAndFiles(backupPath, Path.Combine(targetPath, e.Name));
                            Directory.Delete(e.FullPath, true);
                            LogManager.WriteTraceLog(string.Format("Has Backup And Copy!SourcePath:{0},BackupPath:{1},TargetPath:{2}", e.FullPath, backupPath, targetPath));
                        }
                        catch (Exception ex)
                        {
                            LogManager.WriteErrorLog(ex);
                        }
                    }
                }

            }
            catch (Exception ex)
            {
                LogManager.WriteErrorLog(ex);
            }
        }
    }
}

