using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Gtk;
using System.Net;
using System.Net.Sockets;

namespace MonitoringApp.Code.App
{
    public class CreateMDDir
    {
        public static object obj = new object();
        public static MainWindow mainWindow;
        //		static MessageDialog messageDialog;
        public static void CreateMDDirForInit()
        {
            //Load Configuration
            var configLst = Configuration.ConfigLst;
            if (configLst != null)
            {
                MDDirManager.DeleteBackupExpiredDirs();
                foreach (var myConfig in configLst)
                {
                    #region Check Path
                    if (string.IsNullOrEmpty(myConfig.Path))
                    {
                        LogManager.WriteErrorLog("Not Config \"MonitoringDirectory\" Element's Dir Path.");
                        continue;
                    }
                    if (!Directory.Exists(myConfig.Path))
                    {
                        LogManager.WriteErrorLog(string.Format("Directory '{0}' Not Exist.", myConfig.Path));
                        continue;
                    }
                    #endregion
                    switch (myConfig.MDType)
                    {
                        case MonitoringApp.Custom.MonitoringDirectoryType.Immediate:
                            {
                                CreateImmediateDir(myConfig);
                                var taskTimer = new TaskTimer() { Enabled = true, AutoReset = true, Interval = 60 * 60000, CurrentConfig = myConfig };
                                taskTimer.Elapsed += new System.Timers.ElapsedEventHandler(taskTimer_Elapsed);

                                StartTaskToBackup(taskTimer);
                            }
                            break;
                        case MonitoringApp.Custom.MonitoringDirectoryType.Cycle:
                            {
                                var taskTimer = new TaskTimer() { Enabled = true, AutoReset = true, Interval = myConfig.CycleMinutes * 60000, CurrentConfig = myConfig };
                                taskTimer.Elapsed += new System.Timers.ElapsedEventHandler(taskTimer_Elapsed);
                                break;
                            }
                        default:
                            break;
                    }
                }
            }
            else
            {
                LogManager.WriteErrorLog("Load Configuration Fail,Is NUll.");
                Application.Quit();
            }
        }

        private static void StartTaskToBackup(TaskTimer taskTimer)
        {
            lock (obj)
            {
                MDDirManager.CopyChildrenFolderAndFiles(taskTimer.CurrentConfig.Path, taskTimer.CurrentConfig.BackupPath, taskTimer.CurrentConfig.TargetPath, true);
            }
            LogManager.WriteTraceLog(string.Format("BackUp Has Compelete!Soure:{0},Target:{1},BackUp:{2}.", taskTimer.CurrentConfig.Path, taskTimer.CurrentConfig.TargetPath,
taskTimer.CurrentConfig.BackupPath));
        }

        static void taskTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            TaskTimer taskTimer = sender as TaskTimer;
            if (taskTimer == null)
            {
                LogManager.WriteErrorLog("TaskTimer is NULL.");
                return;
            }
            try
            {
                StartTaskToBackup(taskTimer);
            }
            catch (Exception ex)
            {
                LogManager.WriteErrorLog(ex);
            }
        }


        private static void CreateImmediateDir(Custom.MyConfig myConfig)
        {
            FileSystemWatcher watcher = new FileSystemWatcher();
            watcher.Path = myConfig.Path;
            watcher.EnableRaisingEvents = true;
            watcher.IncludeSubdirectories = myConfig.IncludeSubdirectories;
            watcher.Filter = myConfig.Filter;
            watcher.Created += new FileSystemEventHandler(watcher_Created);
            watcher.Changed += new FileSystemEventHandler(watcher_Changed);
            //watcher.Deleted += new FileSystemEventHandler(watcher_Deleted);
        }

        static void watcher_Deleted(object sender, FileSystemEventArgs e)
        {
        }

        static void watcher_Changed(object sender, FileSystemEventArgs e)
        {
            UploadFilesAndBackUp(e);
        }

        static void watcher_Created(object sender, FileSystemEventArgs e)
        {
            UploadFilesAndBackUp(e);
        }

        private static void UploadFilesAndBackUp(FileSystemEventArgs e)
        {
            try
            {
                if (Configuration.ConfigLst == null)
                    return;
                var myConfig = Configuration.ConfigLst.FirstOrDefault(p => e.FullPath.StartsWith(p.Path));
                if (myConfig == null)
                {
                    LogManager.WriteErrorLog("Can't find MDDir's Path in ConfigLst.myConfig is NULL.");
                    return;
                }

                //                } catch (Exception ex) {
                //					LogManager.WriteErrorLog(ex);
                //                			messageDialog=	new MessageDialog(mainWindow,DialogFlags.Modal,MessageType.Error,ButtonsType.Ok,ex.Message);
                //							messageDialog.Title="Error";
                //							ResponseType respone=(ResponseType)messageDialog.Run();
                //							if (respone==ResponseType.Ok || respone==ResponseType.Close || respone==ResponseType.DeleteEvent) {
                //								messageDialog.Destroy();
                //							}
                //					return;
                //                }
				var backupPath = MDDirManager.GetDirDateString(myConfig.BackupPath);
                var targetPath = MDDirManager.GetDirDateString(myConfig.TargetPath);
                //IF is File
                if (File.Exists(e.FullPath))
                {
                    {
                        try
                        {

                			if (!Directory.Exists(backupPath))
                    			Directory.CreateDirectory(backupPath);

                			if (!Directory.Exists(targetPath))
                   				Directory.CreateDirectory(targetPath);
                            string _backFilePath = Path.Combine(backupPath, e.Name);
                            string _targetFilePath = Path.Combine(targetPath, e.Name);
                            lock (obj)
                            {
                                File.Copy(e.FullPath, _backFilePath, true);
                            }
                            lock (obj)
                            {
                                File.Copy(e.FullPath, _targetFilePath, true);
                            }
                            lock (obj)
                            {
                                File.Delete(e.FullPath);
                            }
                            LogManager.WriteTraceLog(string.Format("Has Backup And Copy!SourcePath:{0},BackupPath:{1},TargetPath:{2}", e.FullPath, _backFilePath, _targetFilePath));
                        }
                        catch (Exception ex)
                        {
                            LogManager.WriteErrorLog(ex);
                        }
                    }
                }
                //if is Directory
                if (Directory.Exists(e.FullPath))
                {
                    lock (obj)
                    {
                        try
                        {
                            lock (obj)
                            {
                                MDDirManager.CopyChildrenFolderAndFiles(e.FullPath, Path.Combine(backupPath, e.Name), Path.Combine(targetPath, e.Name), true);
                            }
                            LogManager.WriteTraceLog(string.Format("Has Backup And Copy!SourcePath:{0},BackupPath:{1},TargetPath:{2}", e.FullPath, backupPath, targetPath));
                        }
                        catch (Exception ex)
                        {
                            LogManager.WriteErrorLog(ex);

                        }
                    }
                }

            }
            catch (Exception ex)
            {
                LogManager.WriteErrorLog(ex);
            }
        }
    }
}

