using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Gtk;

namespace MonitoringApp.Code.App
{
    public class LogManager
    {
        public static object obj = new object();

        static LogManager()
        {
        }


        #region Fields and Properties
        private static string _logPath = string.Empty;
        public static string LogPath
        {
            get
            {
                lock (obj)
                {
                    if (_logPath == string.Empty)
                    {
                        var _logStr = string.Format("{0}/log/{1}/{2}", Path.Combine(Directory.GetCurrentDirectory(), "Data"), DateTime.Now.ToString("yyyy-MM"), DateTime.Now.Day);
                        if (!System.IO.Directory.Exists(_logStr))
                            System.IO.Directory.CreateDirectory(_logStr);
                        return string.Format("{0}/", _logStr);
                    }
                    return _logPath;
                }
            }
            set { _logPath = value; }
        }
        private static string _logFielPrefix = string.Empty;
        public static string LogFielPrefix
        {
            get { return _logFielPrefix; }
            set { _logFielPrefix = value; }
        }
        #endregion

        #region Log

        public static void WriteLog(string logFile, string msg)
        {
            try
            {
                lock (obj)
                {
                    var _sb = new System.Text.StringBuilder();
                    _sb.Append(LogPath).Append(LogFielPrefix).Append(logFile).Append(".Log");
                    using (var _sw = System.IO.File.AppendText(_sb.ToString()))
                    {
                        _sw.WriteLine("==============================={0}===============================", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
                        _sw.WriteLine(msg);
                        _sw.Flush();
                        _sw.Close();
                        Console.WriteLine(msg);

                    }
                }
            }
            catch
            { }
        }

        public static void WriteLog(LogFile logFile, string msg)
        {
            WriteLog(logFile.ToString(), msg);
        }
        public static void WriteTraceLog(string msg)
        {
            Console.WriteLine(msg);
        }

        public static void WriteErrorLog(string msg)
        {
            WriteLog(LogFile.Error, msg);
        }
        public static void WriteErrorLog(Exception ex)
        {

            var _sbError = new StringBuilder();

            _sbError.Append("[Message]").Append(ex.Message);
            _sbError.Append("\r\n[TargetSite]").Append(ex.TargetSite);
            _sbError.Append("\r\n[StackTrace]").Append(ex.StackTrace);
            _sbError.Append("\r\n[Source]").Append(ex.Source);
            foreach (var _item in ex.Data.Keys)
            {
                _sbError.Append("\r\n[").Append(_item).Append("]").Append(ex.Data[_item]);
            }
            if (ex.InnerException != null && ex.InnerException.Message != ex.Message)
            {
                _sbError.AppendLine("____________________________________________________________________________________");
                _sbError.Append("[Message]").Append(ex.InnerException.Message);
                _sbError.Append("\r\n[TargetSite]").Append(ex.InnerException.TargetSite);
                _sbError.Append("\r\n[StackTrace]").Append(ex.InnerException.StackTrace);
                _sbError.Append("\r\n[Source]").Append(ex.InnerException.Source);
                _sbError.AppendLine("____________________________________________________________________________________");
            }
            WriteLog(LogFile.Error, _sbError.ToString());
        }
        public static void WriteWarningLog(string msg)
        {
            WriteLog(LogFile.Warning, msg);
        }
        #endregion
    }

    public enum LogFile
    {
        Trace,
        Warning,
        Error,
        SQL,
        System
    }
}

