using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Timers;

namespace MonitoringApp.Code.App
{
    public class MDDirManager
    {
        static MDDirManager()
        {
            Timer tmr = new Timer() { Interval = 14400000, AutoReset = true, Enabled = true };
            tmr.Elapsed += new ElapsedEventHandler(tmr_Elapsed);
        }

        static void tmr_Elapsed(object sender, ElapsedEventArgs e)
        {
            //Check every 4 hours backup directory is expired, if you perform the removal expired
            DeleteBackupExpiredDirs();
        }
        public static void DeleteBackupExpiredDirs()
        {
            var configLst = Configuration.ConfigLst;
            if (configLst == null)
                return;
            foreach (var myConfig in Configuration.ConfigLst)
            {
                var backupDir = new DirectoryInfo(myConfig.BackupPath);
                TimeSpan ts = DateTime.Now.Subtract(backupDir.LastWriteTime).Duration();//ʱ
                if (ts.Days > myConfig.BackupExpired)
                {
                    DeleteChildrenFolderAndFiles(myConfig.BackupPath);
                }
                else
                {
                    //Cycle sub-folders
                    DirectoryInfo[] subDirArray = backupDir.GetDirectories();
                    foreach (DirectoryInfo subDir in subDirArray)
                    {
                        DeleteBackupExpiredDirs(subDir, myConfig);
                    }
                }
            }
        }
        public static void DeleteBackupExpiredDirs(DirectoryInfo dir, Custom.MyConfig myConfig)
        {
            TimeSpan ts = DateTime.Now.Subtract(dir.LastWriteTime).Duration();//ʱ
            if (ts.Days > myConfig.BackupExpired)
            {
                DeleteChildrenFolderAndFiles(myConfig.BackupPath);
            }
            else
            {
                //Cycle sub-folders
                DirectoryInfo[] subDirArray = dir.GetDirectories();
                foreach (DirectoryInfo subDir in subDirArray)
                {
                    DeleteBackupExpiredDirs(subDir, myConfig);
                }
            }
        }
        public static void CopyFolder(string sPath, string dPath)
        {
            DirectoryInfo sDir = new DirectoryInfo(sPath);
            string targetPath = Path.Combine(dPath, sDir.Name);
            //Create Target directory
            if (!Directory.Exists(targetPath))
                Directory.CreateDirectory(targetPath);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            foreach (FileInfo file in fileArray)
            {
                file.CopyTo(Path.Combine(targetPath, file.Name), true);
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            foreach (DirectoryInfo subDir in subDirArray)
            {
                CopyFolder(subDir.FullName, Path.Combine(targetPath, subDir.Name));
            }
        }
        public static void CopyChildrenFolderAndFiles(string sPath, string dPath)
        {
            if (!Directory.Exists(dPath))
                Directory.CreateDirectory(dPath);
            DirectoryInfo sDir = new DirectoryInfo(sPath);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            foreach (FileInfo file in fileArray)
            {
                file.CopyTo(Path.Combine(dPath, file.Name), true);
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            foreach (DirectoryInfo subDir in subDirArray)
            {
                CopyFolder(subDir.FullName, Path.Combine(dPath, subDir.Name));
            }
        }
        public static void DeleteChildrenFolderAndFiles(string path)
        {
            if (!Directory.Exists(path))
                return;
            DirectoryInfo sDir = new DirectoryInfo(path);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            foreach (FileInfo file in fileArray)
            {
                file.Delete();
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            foreach (DirectoryInfo subDir in subDirArray)
            {
                subDir.Delete(true);
            }
        }
    }
}