using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Timers;

namespace MonitoringApp.Code.App
{
    public class MDDirManager
    {
        static MDDirManager()
        {
            Timer tmr = new Timer() { Interval = 14400000, AutoReset = true, Enabled = true };
            tmr.Elapsed += new ElapsedEventHandler(tmr_Elapsed);
        }

        static void tmr_Elapsed(object sender, ElapsedEventArgs e)
        {
            //Check every 4 hours backup directory is expired, if you perform the removal expired
            DeleteBackupExpiredDirs();
        }
        public static void DeleteBackupExpiredDirs()
        {
            var configLst = Configuration.ConfigLst;
            if (configLst == null)
                return;
            foreach (var myConfig in Configuration.ConfigLst)
            {
                if (!Directory.Exists(myConfig.BackupPath))
                    continue;
                var backupDir = new DirectoryInfo(myConfig.BackupPath.Replace("{Y}", string.Empty).Replace("{M}", string.Empty).Replace("{D}", string.Empty).TrimEnd('\\'));
                TimeSpan ts = DateTime.Now.Subtract(backupDir.LastWriteTime).Duration();//ʱ
                if (ts.Days > myConfig.BackupExpired)
                {
                    DeleteChildrenFolderAndFiles(myConfig.BackupPath);
                }
                else
                {
                    //Cycle sub-folders
                    DirectoryInfo[] subDirArray = backupDir.GetDirectories();
                    foreach (DirectoryInfo subDir in subDirArray)
                    {
                        DeleteBackupExpiredDirs(subDir, myConfig);
                    }
                }
            }
        }
        public static void DeleteBackupExpiredDirs(DirectoryInfo dir, Custom.MyConfig myConfig)
        {
            TimeSpan ts = DateTime.Now.Subtract(dir.LastWriteTime).Duration();//ʱ
            if (ts.Days > myConfig.BackupExpired)
            {
                DeleteChildrenFolderAndFiles(myConfig.BackupPath);
            }
            else
            {
                //Cycle sub-folders
                DirectoryInfo[] subDirArray = dir.GetDirectories();
                foreach (DirectoryInfo subDir in subDirArray)
                {
                    DeleteBackupExpiredDirs(subDir, myConfig);
                }
            }
        }
        public static void CopyFolder(string sPath, string dPath)
        {
            DirectoryInfo sDir = new DirectoryInfo(sPath);
            string targetPath = Path.Combine(dPath, sDir.Name);
            //Create Target directory
            if (!Directory.Exists(targetPath))
                Directory.CreateDirectory(targetPath);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            foreach (FileInfo file in fileArray)
            {
                file.CopyTo(Path.Combine(targetPath, file.Name), true);
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            foreach (DirectoryInfo subDir in subDirArray)
            {
                CopyFolder(subDir.FullName, Path.Combine(targetPath, subDir.Name));
            }
        }

        public static void CopyChildrenFolderAndFiles(string sPath, string dPath, bool isDeleteAfterCopy)
        {
            if (!Directory.Exists(dPath))
                Directory.CreateDirectory(dPath);
            DirectoryInfo sDir = new DirectoryInfo(sPath);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            for (int i = 0; i < fileArray.Length; i++)
            {
                fileArray[i].CopyTo(Path.Combine(dPath, fileArray[i].Name), true);
                if (isDeleteAfterCopy)
                    fileArray[i].Delete();
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            for (int i = 0; i < subDirArray.Length; i++)
            {
                CopyFolder(subDirArray[i].FullName, Path.Combine(dPath, subDirArray[i].Name));
                if (isDeleteAfterCopy)
                    subDirArray[i].Delete(true);
            }
        }

        public static void CopyChildrenFolderAndFiles(string sPath, string backupPath, string targetPath, bool isDeleteAfterCopy)
        {
            backupPath = GetDirDateString(backupPath);
            targetPath = GetDirDateString(targetPath);
            if (!Directory.Exists(backupPath))
                Directory.CreateDirectory(backupPath);
            if (!Directory.Exists(targetPath))
                Directory.CreateDirectory(targetPath);
            DirectoryInfo sDir = new DirectoryInfo(sPath);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            for (int i = 0; i < fileArray.Length; i++)
            {
                if (fileArray.Length < 2000)
                    fileArray[i].CopyTo(Path.Combine(backupPath, fileArray[i].Name), true);
                fileArray[i].CopyTo(Path.Combine(targetPath, fileArray[i].Name), true);
                if (isDeleteAfterCopy)
                    fileArray[i].Delete();
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            for (int i = 0; i < subDirArray.Length; i++)
            {
                CopyFolder(subDirArray[i].FullName, Path.Combine(backupPath, subDirArray[i].Name));
                CopyFolder(subDirArray[i].FullName, Path.Combine(targetPath, subDirArray[i].Name));
                if (isDeleteAfterCopy)
                    subDirArray[i].Delete(true);
            }
        }

        public static string GetDirDateString(string path)
        {
            return path.Replace("{Y}", DateTime.Now.Year.ToString()).Replace("{M}", DateTime.Now.Month.ToString()).Replace("{D}", DateTime.Now.Day.ToString());
        }
        public static void DeleteChildrenFolderAndFiles(string path)
        {
            if (!Directory.Exists(path))
                return;
            DirectoryInfo sDir = new DirectoryInfo(path);
            //Copy Files
            FileInfo[] fileArray = sDir.GetFiles();
            foreach (FileInfo file in fileArray)
            {
                file.Delete();
            }
            //Cycle sub-folders
            DirectoryInfo[] subDirArray = sDir.GetDirectories();
            foreach (DirectoryInfo subDir in subDirArray)
            {
                subDir.Delete(true);
            }
        }
    }
}